##*************************************************************************##
##  SCALASCA    http://www.scalasca.org/                                   ##
##*************************************************************************##
##  Copyright (c) 2014-2024                                                ##
##  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          ##
##                                                                         ##
##  This software may be modified and distributed under the terms of       ##
##  a BSD-style license.  See the COPYING file in the package base         ##
##  directory for details.                                                 ##
##*************************************************************************##


# STT_COMPILE_FLAGS_NOWARN_OMP_PRAGMAS
# ------------------------------------
# Determines a suitable compiler option (for the currently selected language)
# to suppress warnings about unknown OpenMP pragmas when compiling with OpenMP
# support disabled.
#
# NOTES:
#   - This macro currently only works for C and C++.
#   - The selected compiler option may turn off all pragma warnings, not
#     only OpenMP-specific ones.
#
# List of provided automake substitutions:
#  `NOWARN_OMP_PRAGMAS_langFLAGS`::     Selected compiler option
#                                       (`lang` is either CC or CXX)
#
AC_DEFUN([STT_COMPILE_FLAGS_NOWARN_OMP_PRAGMAS],
[
AC_MSG_CHECKING([for $[]_AC_CC[] option to suppress OpenMP pragma warnings])

dnl Try to compile w/o additional option first
AFS_COMPILE_IFELSE_WERROR([_STT_LANG_OMP_PRAGMA],
    [_stt_compile_flags_nowarn_omp_pragmas='none needed']
    [_stt_compile_flags_nowarn_omp_pragmas='needed'])

dnl If compilation failed, try these flags:
dnl   Intel >= 14        -diag-disable 3180  (disables OpenMP pragma warnings only)
dnl   Intel <  14        -diag-disable 161   (disables all pragma warnings)
dnl   GCC                -Wno-unknown-pragmas
dnl
AS_IF([test "$_stt_compile_flags_nowarn_omp_pragmas" = 'needed'],
    [_stt_compile_flags_nowarn_omp_pragmas='unsupported'
     for stt_option in "-diag-disable 3180" "-diag-disable 161" \
                       "-Wno-unknown-pragmas"; do
         _stt_save_[]_AC_LANG_PREFIX[]FLAGS=$[]_AC_LANG_PREFIX[]FLAGS
         _AC_LANG_PREFIX[]FLAGS="$[]_AC_LANG_PREFIX[]FLAGS $stt_option"
         AFS_COMPILE_IFELSE_WERROR([_STT_LANG_OMP_PRAGMA],
             [_stt_compile_flags_nowarn_omp_pragmas=$stt_option])
         _AC_LANG_PREFIX[]FLAGS=$_stt_save_[]_AC_LANG_PREFIX[]FLAGS
         AS_IF([test "$_stt_compile_flags_nowarn_omp_pragmas" != 'unsupported'],
            [break])
     done])
AC_MSG_RESULT([$_stt_compile_flags_nowarn_omp_pragmas])

dnl Substitute output variable
AS_CASE([$_stt_compile_flags_nowarn_omp_pragmas],
    ["none needed" | unsupported],
        [AS_UNSET([NOWARN_OMP_PRAGMAS_[]_AC_LANG_PREFIX[]FLAGS])],
    [NOWARN_OMP_PRAGMAS_[]_AC_LANG_PREFIX[]FLAGS=$_stt_compile_flags_nowarn_omp_pragmas])
AC_SUBST([NOWARN_OMP_PRAGMAS_]_AC_LANG_PREFIX[FLAGS])
])


# _STT_LANG_OMP_PRAGMA
# --------------------
# Expands to some language dependent source code for testing whether OpenMP
# pragmas issue a compiler warning.
#
AC_DEFUN([_STT_LANG_OMP_PRAGMA],
[AC_LANG_SOURCE([_AC_LANG_DISPATCH([$0], _AC_LANG, $@)])])


# _STT_LANG_OMP_PRAGMA(C)
# -----------------------
# Test source for C.
#
m4_define([_STT_LANG_OMP_PRAGMA(C)],
[
int main () {
    #pragma omp parallel
    {}
    return 0;
}
])


# _STT_LANG_OMP_PRAGMA(C++)
# -------------------------
# Use C test source also for C++.
#
m4_copy([_STT_LANG_OMP_PRAGMA(C)], [_STT_LANG_OMP_PRAGMA(C++)])
