##*************************************************************************##
##  SCALASCA    http://www.scalasca.org/                                   ##
##*************************************************************************##
##  Copyright (c) 2021                                                     ##
##  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          ##
##                                                                         ##
##  This software may be modified and distributed under the terms of       ##
##  a BSD-style license.  See the COPYING file in the package base         ##
##  directory for details.                                                 ##
##*************************************************************************##


# STT_CXX_COMPILE_MPI2
# --------------------
# Checks whether the MPI C++ compiler wrapper is able to compile a test
# program using a selected set of MPI 2.0 functions, and aborts if this
# is not the case.  Note that this test is not meant to be exhaustive,
# but more as a non-trivial sanity check.
#
# List of configure variables set:
#   `stt_cv_cxx_compile_mpi2`::  'yes' if MPI 2.0 is supported, error
#                                out otherwise
#
AC_DEFUN([STT_CXX_COMPILE_MPI2], [
AC_LANG_PUSH([C++])
AC_CACHE_CHECK([whether $CXX supports MPI 2.0],
    [stt_cv_cxx_compile_mpi2],
    [AC_COMPILE_IFELSE(
        [AC_LANG_PROGRAM(
            [_STT_CXX_COMPILE_MPI_testheader],
            [_STT_CXX_COMPILE_MPI_testbody_2_0])],
        [stt_cv_cxx_compile_mpi2=yes],
        [AC_MSG_FAILURE([$CXX cannot compile MPI 2.0 programs])])
    ])
AC_LANG_POP([C++])
])


m4_define([_STT_CXX_COMPILE_MPI_testheader],
[[
#include <cstddef>
#include <mpi.h>
]])


m4_define([_STT_CXX_COMPILE_MPI_testbody_2_0],
[[
    // Check for MPI_Init_thread
    int provided;
    MPI_Init_thread(NULL, NULL, MPI_THREAD_FUNNELED, &provided);

    // Check for MPI_Exscan
    int sendBuf = 1;
    int recvBuf = 0;
    MPI_Exscan(&sendBuf, &recvBuf, 1, MPI_INT, MPI_SUM, MPI_COMM_WORLD);
    // recvBuf == rank

    // Check for MPI_IN_PLACE
    recvBuf = 1;
    MPI_Allreduce(MPI_IN_PLACE, &recvBuf, 1, MPI_INT, MPI_SUM, MPI_COMM_WORLD);
    // recvBuf == size

    // Check for one-sided communication
    recvBuf = 0;
    MPI_Win window;
    MPI_Win_create(&recvBuf, sizeof(int), sizeof(int), MPI_INFO_NULL,
                   MPI_COMM_WORLD, &window);
    MPI_Group group;
    MPI_Comm_group(MPI_COMM_WORLD, &group);

    MPI_Win_fence(MPI_MODE_NOPRECEDE, window);
    MPI_Accumulate(&sendBuf, 1, MPI_INT, 0, 0, 1, MPI_INT, MPI_SUM, window);
    MPI_Win_fence(0, window);
    // recvBuf@r0 == size

    MPI_Win_post(group, MPI_MODE_NOPUT, window);
    MPI_Win_start(group, 0, window);
    MPI_Get(&sendBuf, 1, MPI_INT, 0, 0, 1, MPI_INT, window);
    MPI_Win_complete(window);
    MPI_Win_wait(window);
    // sendBuf = size

    MPI_Group_free(&group);
    MPI_Win_free(&window);

    MPI_Finalize();
]])
