/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2024                                                     **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef PEARL_COMMDESTROY_REP_H
#define PEARL_COMMDESTROY_REP_H


#include <cstdint>

#include <pearl/Event_rep.h>


/*-------------------------------------------------------------------------*/
/**
 *  @file    CommDestroy_rep.h
 *  @ingroup PEARL_base
 *  @brief   Declaration of the class CommDestroy_rep.
 *
 *  This header file provides the declaration of the class CommDestroy_rep.
 **/
/*-------------------------------------------------------------------------*/


namespace pearl
{
/*-------------------------------------------------------------------------*/
/**
 *  @class   CommDestroy_rep
 *  @ingroup PEARL_base
 *  @brief   %Event representation for communicator destruction operations.
 **/
/*-------------------------------------------------------------------------*/

class PEARL_NOPAD_ATTR CommDestroy_rep
    : public Event_rep
{
    public:
        /// @name Constructors & destructor
        /// @{

        explicit
        CommDestroy_rep(timestamp_t         timestamp,
                        const Communicator& communicator);

        CommDestroy_rep(const GlobalDefs& defs,
                        Buffer&           buffer);

        /// @}
        /// @name Event type information
        /// @{

        virtual event_t
        getType() const;

        virtual bool
        isOfType(event_t type) const;

        /// @}
        /// @name Access event data
        /// @{

        virtual const Communicator&
        getComm() const;

        /// @}


    protected:
        /// @name Serialize event data
        /// @{

        virtual void
        pack(Buffer& buffer) const;

        /// @}
        /// @name Generate human-readable output of event data
        /// @{

        virtual std::ostream&
        output(std::ostream& stream) const;

        /// @}


    private:
        /// Reference to Communicator definition
        const Communicator& mCommunicator;
};
}    // namespace pearl


#endif    // !PEARL_COMMDESTROY_REP_H
