/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2013                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  Copyright (c) 2023                                                     **
**  RWTH Aachen University, IT Center                                      **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef PEARL_MPIRECEIVE_REP_H
#define PEARL_MPIRECEIVE_REP_H


#include <cstdint>

#include <pearl/Event_rep.h>


/*-------------------------------------------------------------------------*/
/**
 *  @file    MpiReceive_rep.h
 *  @ingroup PEARL_mpi
 *  @brief   Declaration of the class MpiReceive_rep.
 *
 *  This header file provides the declaration of the class MpiReceive_rep.
 **/
/*-------------------------------------------------------------------------*/


namespace pearl
{
/*-------------------------------------------------------------------------*/
/**
 *  @class   MpiReceive_rep
 *  @ingroup PEARL_mpi
 *  @brief   %Event representation for MPI_RECV events.
 **/
/*-------------------------------------------------------------------------*/

class PEARL_NOPAD_ATTR MpiReceive_rep
    : public Event_rep
{
    public:
        /// @name Constructors & destructor
        /// @{

        MpiReceive_rep(timestamp_t         timestamp,
                       const Communicator& communicator,
                       std::uint32_t       source,
                       std::uint32_t       tag,
                       std::uint64_t       bytesReceived);

        MpiReceive_rep(const GlobalDefs& defs,
                       Buffer&           buffer);

        /// @}
        /// @name Event type information
        /// @{

        virtual event_t
        getType() const;

        virtual bool
        isOfType(event_t type) const;

        /// @}
        /// @name Access event data
        /// @{

        virtual const Communicator&
        getComm() const;

        virtual std::uint32_t
        getTag() const;

        virtual std::uint64_t
        getBytesReceived() const;

        virtual std::uint32_t
        getSource() const;

        virtual std::uint64_t
        getRequestId() const;

        /// @}


    protected:
        /// @name Serialize event data
        /// @{

        virtual void
        pack(Buffer& buffer) const;

        /// @}
        /// @name Generate human-readable output of event data
        /// @{

        virtual std::ostream&
        output(std::ostream& stream) const;

        /// @}
        /// @name Find previous/next communication request entries
        /// @{

        virtual std::uint32_t
        getNextReqoffs() const;

        virtual std::uint32_t
        getPrevReqoffs() const;

        virtual void
        setPrevReqoffs(std::uint32_t offset);

        virtual void
        setNextReqoffs(std::uint32_t offset);

        /// @}


    private:
        /// MPI communicator used
        const Communicator& mCommunicator;

        /// Local rank of sender location w.r.t. communicator
        std::uint32_t mSource;

        /// Message tag
        std::uint32_t mTag;

        /// Message length
        std::uint64_t mBytesReceived;
};
}    // namespace pearl


#endif    // !PEARL_MPIRECEIVE_REP_H
