/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2022                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2013                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  Copyright (c) 2023                                                     **
**  RWTH Aachen University, IT Center                                      **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef PEARL_MPISENDCOMPLETE_REP_H
#define PEARL_MPISENDCOMPLETE_REP_H


#include <cstdint>

#include <pearl/Event_rep.h>


/*-------------------------------------------------------------------------*/
/**
 *  @file    MpiSendComplete_rep.h
 *  @ingroup PEARL_mpi
 *  @brief   Declaration of the class MpiSendComplete_rep.
 *
 *  This header file provides the declaration of the class MpiSendComplete_rep.
 **/
/*-------------------------------------------------------------------------*/


namespace pearl
{
/*-------------------------------------------------------------------------*/
/**
 *  @class   MpiSendComplete_rep
 *  @ingroup PEARL_mpi
 *  @brief   %Event representation for MPI_SEND_COMPLETE events.
 **/
/*-------------------------------------------------------------------------*/

class PEARL_NOPAD_ATTR MpiSendComplete_rep
    : public Event_rep
{
    public:
        /// @name Constructors & destructor
        /// @{

        MpiSendComplete_rep(timestamp_t   timestamp,
                            std::uint64_t requestId);

        MpiSendComplete_rep(const GlobalDefs& defs,
                            Buffer&           buffer);

        /// @}
        /// @name Event type information
        /// @{

        virtual event_t
        getType() const;

        virtual bool
        isOfType(event_t type) const;

        /// @}
        /// @name Access event data
        /// @{

        virtual std::uint64_t
        getRequestId() const;

        /// @}
        /// @name Modify event data
        /// @{

        virtual void
        setRequestId(std::uint64_t requestId);

        /// @}


    protected:
        /// @name Generate human-readable output of event data
        /// @{

        virtual std::ostream&
        output(std::ostream& stream) const;

        /// @}
        /// @name Find previous/next communication request entries
        /// @{

        virtual std::uint32_t
        getNextReqoffs() const;

        virtual std::uint32_t
        getPrevReqoffs() const;

        virtual void
        setPrevReqoffs(std::uint32_t offset);

        virtual void
        setNextReqoffs(std::uint32_t offset);

        /// @}


    private:
        /// Non-blocking request ID
        std::uint64_t mRequestId;

        /// Offset to previous (test or request) event
        std::uint32_t mPrevReqoffs;
};
}    // namespace pearl


#endif    // !PEARL_MPISENDCOMPLETE_REP_H
