/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2024                                                     **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef PEARL_NONBLOCKINGCOLLREQUEST_REP_H
#define PEARL_NONBLOCKINGCOLLREQUEST_REP_H


#include <cstdint>

#include <pearl/Event_rep.h>


/*-------------------------------------------------------------------------*/
/**
 *  @file    NonBlockingCollRequest_rep.h
 *  @ingroup PEARL_base
 *  @brief   Declaration of the class NonBlockingCollRequest_rep.
 *
 *  This header file provides the declaration of the class
 *  NonBlockingCollRequest_rep.
 **/
/*-------------------------------------------------------------------------*/


namespace pearl
{
/*-------------------------------------------------------------------------*/
/**
 *  @class   NonBlockingCollRequest_rep
 *  @ingroup PEARL_base
 *  @brief   %Event representation for NON_BLOCKING_COLLECTIVE_REQUEST
 *           events.
 **/
/*-------------------------------------------------------------------------*/

class PEARL_NOPAD_ATTR NonBlockingCollRequest_rep
    : public Event_rep
{
    public:
        /// @name Constructors & destructor
        /// @{

        explicit
        NonBlockingCollRequest_rep(timestamp_t   timestamp,
                                   std::uint64_t requestId);

        NonBlockingCollRequest_rep(const GlobalDefs& defs,
                                   Buffer&           buffer);

        /// @}
        /// @name Event type information
        /// @{

        virtual event_t
        getType() const;

        virtual bool
        isOfType(event_t type) const;

        /// @}
        /// @name Access event data
        /// @{

        virtual std::uint64_t
        getRequestId() const;

        /// @}


    protected:
        /// @name Generate human-readable output of event data
        /// @{

        virtual std::ostream&
        output(std::ostream& stream) const;

        /// @}
        /// @name Find previous/next communication request entries
        /// @{

        virtual std::uint32_t
        getNextReqoffs() const;

        virtual std::uint32_t
        getPrevReqoffs() const;

        virtual void
        setPrevReqoffs(std::uint32_t offset);

        virtual void
        setNextReqoffs(std::uint32_t offset);

        /// @}


    private:
        /// Non-blocking request ID
        std::uint64_t mRequestId;

        /// Offset to next test or non-blocking collective complete event
        std::uint32_t mNextReqoffs;
};
}    // namespace pearl


#endif    // !PEARL_NONBLOCKINGCOLLREQUEST_REP_H
