/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2022                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2013                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


/*-------------------------------------------------------------------------*/
/**
 *  @file
 *  @ingroup PEARL_base
 *  @brief   Declaration of the class RemoteEventSet.
 *
 *  This header file provides the declaration of the class RemoteEventSet.
 **/
/*-------------------------------------------------------------------------*/


#ifndef PEARL_REMOTEEVENTSET_H
#define PEARL_REMOTEEVENTSET_H


#include <cstdint>
#include <map>
#include <vector>

#include <pearl/RemoteEvent.h>


namespace pearl
{
// --- Forward declarations -------------------------------------------------

class Buffer;
class GlobalDefs;


/*-------------------------------------------------------------------------*/
/**
 *  @class   RemoteEventSet
 *  @ingroup PEARL_base
 *  @brief   Container class for a set of remote events with associated roles.
 **/
/*-------------------------------------------------------------------------*/

class RemoteEventSet
{
    public:
        // --- Public methods ------------------------------

        /// @name Adding/updating events
        /// @{

        void
        addEvent(const RemoteEvent& event,
                 std::uint32_t      key);

        void
        updateEvent(const RemoteEvent& event,
                    std::uint32_t      key);

        /// @}
        /// @name Retrieving events
        /// @{

        RemoteEvent
        getEvent(std::uint32_t key) const;

        /// @}
        /// @name Removing events
        /// @{

        void
        clear();

        /// @}
        /// @name Deserializing data
        /// @{

        void
        unpack(const GlobalDefs& defs,
               Buffer&           buffer);

        /// @}


    private:
        // --- Type definitions ----------------------------

        /// Container type for role |-@> index mapping
        typedef std::map< std::uint32_t, std::uint32_t > KeyMap;

        /// Container type for remote events
        typedef std::vector< RemoteEvent > EventList;


        // --- Data members --------------------------------

        /// Mapping table key (role ID) |-@> index
        KeyMap mKeys;

        /// Current set of events
        EventList mEvents;


        // --- Private methods -----------------------------

        std::uint32_t
        storeEvent(const RemoteEvent& event);
};
}    // namespace pearl


#endif    // !PEARL_REMOTEEVENTSET_H
