/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2013                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  Copyright (c) 2022-2023                                                **
**  RWTH Aachen University, IT Center                                      **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


/*-------------------------------------------------------------------------*/
/**
 *  @file
 *  @ingroup PEARL_mpi
 *  @brief   Implementation of the class MpiRmaCollEnd_rep.
 *
 *  This file provides the implementation of the class MpiRmaCollEnd_rep.
 **/
/*-------------------------------------------------------------------------*/


#include <config.h>

#include <pearl/MpiRmaCollEnd_rep.h>

#include <iostream>

#include <pearl/Buffer.h>
#include <pearl/GlobalDefs.h>
#include <pearl/RmaWindow.h>

#include "iomanip_detail.h"

using namespace std;
using namespace pearl;
using namespace pearl::detail;


// --- Constructors & destructor --------------------------------------------

MpiRmaCollEnd_rep::MpiRmaCollEnd_rep(timestamp_t      timestamp,
                                     const RmaWindow& window)
    : Event_rep(timestamp),
      mWindow(window)
{
}


MpiRmaCollEnd_rep::MpiRmaCollEnd_rep(const GlobalDefs& defs,
                                     Buffer&           buffer)
    : Event_rep(defs, buffer),
      mWindow(defs.getRmaWindow(buffer.get_uint32()))
{
}


// --- Event type information -----------------------------------------------

event_t
MpiRmaCollEnd_rep::getType() const
{
    return MPI_RMA_COLLECTIVE_END;
}


bool
MpiRmaCollEnd_rep::isOfType(event_t type) const
{
    return (  (type == MPI_RMA_COLLECTIVE_END)
           || (type == GROUP_END)
           || (type == GROUP_ALL));
}


// --- Access event data ----------------------------------------------------

const RmaWindow&
MpiRmaCollEnd_rep::getWindow() const
{
    return mWindow;
}


// --- Serialize event data (protected) -------------------------------------

void
MpiRmaCollEnd_rep::pack(Buffer& buffer) const
{
    Event_rep::pack(buffer);

    buffer.put_uint32(mWindow.getId());
}


// --- Generate human-readable output of event data (protected) -------------

ostream&
MpiRmaCollEnd_rep::output(ostream& stream) const
{
    Event_rep::output(stream);

    // Print data
    stream << sep
           << "window: " << mWindow;

    return stream;
}
