/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2024                                                     **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


/*-------------------------------------------------------------------------*/
/**
 *  @file
 *  @ingroup PEARL_base
 *  @brief   Implementation of the class CommCreate_rep.
 *
 *  This file provides the implementation of the class CommCreate_rep.
 **/
/*-------------------------------------------------------------------------*/


#include <config.h>

#include <pearl/CommCreate_rep.h>

#include <ostream>

#include <pearl/Buffer.h>
#include <pearl/Communicator.h>
#include <pearl/GlobalDefs.h>

#include "iomanip_detail.h"

using namespace std;
using namespace pearl;
using namespace pearl::detail;


// --- Constructors & destructor --------------------------------------------

CommCreate_rep::CommCreate_rep(const timestamp_t   timestamp,
                               const Communicator& communicator)
    : Event_rep(timestamp),
      mCommunicator(communicator)
{
}


CommCreate_rep::CommCreate_rep(const GlobalDefs& defs,
                               Buffer&           buffer)
    : Event_rep(defs, buffer),
      mCommunicator(defs.getCommunicator(buffer.get_uint32()))
{
}


// --- Event type information -----------------------------------------------

event_t
CommCreate_rep::getType() const
{
    return COMM_CREATE;
}


bool
CommCreate_rep::isOfType(event_t type) const
{
    return (  (type == COMM_CREATE)
           || (type == GROUP_ALL));
}


const Communicator&
CommCreate_rep::getComm() const
{
    return mCommunicator;
}


// --- Serialize event data (protected) -------------------------------------

void
CommCreate_rep::pack(Buffer& buffer) const
{
    Event_rep::pack(buffer);

    buffer.put_uint32(mCommunicator.getId());
}


// --- Generate human-readable output of event data (protected) -------------

ostream&
CommCreate_rep::output(ostream& stream) const
{
    Event_rep::output(stream);

    // Print data
    stream << sep
           << "comm: " << mCommunicator;

    return stream;
}
