/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2022                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2013                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include <config.h>

#include <pearl/RemoteEventSet.h>

#include <string>

#include <UTILS_Error.h>

#include <pearl/Buffer.h>
#include <pearl/Error.h>

using namespace std;
using namespace pearl;


// --- Adding/updating events -----------------------------------------------

void
RemoteEventSet::addEvent(const RemoteEvent& event,
                         uint32_t           key)
{
    const auto index = storeEvent(event);

    auto result = mKeys.emplace(key, index);

    UTILS_ASSERT(result.second == true);
}


void
RemoteEventSet::updateEvent(const RemoteEvent& event,
                            uint32_t           key)
{
    const auto index = storeEvent(event);

    mKeys[key] = index;
}


// --- Retrieving events ----------------------------------------------------

RemoteEvent
RemoteEventSet::getEvent(uint32_t key) const
{
    KeyMap::const_iterator it = mKeys.find(key);
    if (it == mKeys.end())
    {
        throw RuntimeError("RemoteEventSet::getEvent(uint32_t) "
                           "-- Unknown key: " + std::to_string(key));
    }

    return mEvents[it->second];
}


// --- Removing events ------------------------------------------------------

void
RemoteEventSet::clear()
{
    mKeys.clear();
    mEvents.clear();
}


// --- Deserializing data ---------------------------------------------------

void
RemoteEventSet::unpack(const GlobalDefs& defs,
                       Buffer&           buffer)
{
    const uint32_t keyCount = buffer.get_uint32();
    for (uint32_t i = 0; i < keyCount; ++i)
    {
        const uint32_t key   = buffer.get_uint32();
        const uint32_t index = buffer.get_uint32();

        mKeys.emplace(key, index);
    }

    const uint32_t eventCount = buffer.get_uint32();
    for (uint32_t i = 0; i < eventCount; ++i)
    {
        RemoteEvent event = buffer.get_event(defs);

        mEvents.push_back(event);
    }
}


// --- Private methods ------------------------------------------------------

uint32_t
RemoteEventSet::storeEvent(const RemoteEvent& event)
{
    const auto index = mEvents.size();

    mEvents.push_back(event);

    return index;
}
