/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2022-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


/*-------------------------------------------------------------------------*/
/**
 *  @file
 *  @ingroup PEARL_base
 *  @brief   Declaration of PEARL-internal debug functionality.
 *
 *  This header file provides the declaration of PEARL-internal functions to
 *  initialize and finalize debug output redirection to per-process files.
 **/
/*-------------------------------------------------------------------------*/


#ifndef PEARL_PEARL_DEBUG_H
#define PEARL_PEARL_DEBUG_H


namespace pearl
{
namespace detail
{
// --- Function prototypes -------------------------------------------------

/// @name Debugging functions
/// @{

/// @brief Initialize debug output redirection.
///
/// Initializes the debug output redirection to process-local files if the
/// environment variable `SCALASCA_DEBUG_LOG` is set.  The value of this
/// variable determines the file name prefix, with the global rank -- or
/// zero in case of a single-process run -- appended.
///
void
debugInit();

/// @brief Finalize debug output redirection.
///
/// Finalizes debug output redirection to process-local files if enabled.
/// All following debug output will be printed to `stdout`.
///
/// @thread_safety
///     This function is thread-safe.
///
void
debugFinalize();

/// @}
}    // namespace pearl::detail
}    // namespace pearl


#endif    // !PEARL_PEARL_DEBUG_H
