/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2023                                                     **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


/*-------------------------------------------------------------------------*/
/**
 *  @file
 *  @ingroup PEARL_base
 *  @brief   Declaration of PEARL-internal error handling functionality.
 *
 *  This header file provides the declaration of PEARL-internal functions
 *  to initialize and finalize a custom error handling.
 **/
/*-------------------------------------------------------------------------*/


#ifndef PEARL_PEARL_ERROR_H
#define PEARL_PEARL_ERROR_H


namespace pearl
{
namespace detail
{
// --- Function prototypes -------------------------------------------------

/// @name Error handling functions
/// @{

/// @brief Initialize custom error handling.
///
/// Registers a custom error handler with the exception handling module of
/// the common utils library.
///
void
errorInit();

/// @brief Finalize custom error handling.
///
/// Deregisters the custom error handler from the exception handling module
/// of the common utils library.  Afterwards, the default handler will be
/// used again.
///
void
errorFinalize();

/// @}
}    // namespace pearl::detail
}    // namespace pearl


#endif    // !PEARL_PEARL_ERROR_H
