/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2022                                                     **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef SCOUT_WAITSTATECANDIDATES_H
    #error "Do not include this file!  Use 'WaitstateCandidates.h' instead."
#endif


#include "CbData.h"


namespace scout
{
// --------------------------------------------------------------------------
//
//  WaitstateCandidate & related functions
//
// --------------------------------------------------------------------------

// --- Constructors & destructor --------------------------------------------

inline
WaitstateCandidate::WaitstateCandidate(const pearl::timestamp_t idle,
                                       const pearl::timestamp_t completion)
    : mIdle(idle),
      mCompletion(completion)
{
}


// --- Notification functions -----------------------------------------------

inline void
WaitstateCandidate::announce(const pearl::CallbackManager& cbmanager,
                             const pearl::Event&           event,
                             CbData* const                 data) const
{
    notifyCandidate(cbmanager, event, data);
}


inline void
WaitstateCandidate::confirm(const pearl::CallbackManager& cbmanager,
                            const pearl::Event&           event,
                            CbData* const                 data) const
{
    const auto saveIdle       = data->mIdle;
    const auto saveCompletion = data->mCompletion;

    data->mIdle       = mIdle;
    data->mCompletion = mCompletion;

    notifyInstance(cbmanager, event, data);

    data->mIdle       = saveIdle;
    data->mCompletion = saveCompletion;
}


// --- Comparison operators -------------------------------------------------

inline bool
operator<(const WaitstateCandidate& lhs,
          const WaitstateCandidate& rhs) noexcept
{
    return (lhs.mIdle < rhs.mIdle);
}


// --------------------------------------------------------------------------
//
//  LateSenderCandidate
//
// --------------------------------------------------------------------------

// --- Constructors & destructor --------------------------------------------

inline
LateSenderCandidate::LateSenderCandidate(const pearl::timestamp_t  idle,
                                         const pearl::timestamp_t  completion,
                                         const pearl::RemoteEvent& send,
                                         const pearl::Event&       recv)
    : WaitstateCandidate(idle, completion),
      mSend(send),
      mRecv(recv)
{
}


// --------------------------------------------------------------------------
//
//  LswoCandidate
//
// --------------------------------------------------------------------------

// --- Constructors & destructor --------------------------------------------

inline
LswoCandidate::LswoCandidate(const pearl::timestamp_t  idle,
                             const pearl::timestamp_t  completion,
                             const pearl::RemoteEvent& send,
                             const pearl::Event&       recv)
    : LateSenderCandidate(idle, completion, send, recv)
{
}
}    // namespace scout
